`timescale 1ns / 1ps

////////////////////////////////////////////////////////////////////////////////
// Company:
// Engineer:
//
// Create Date:   16:03:27 07/10/2012
// Design Name:   LcdMdl
// Module Name:   lcdmdl.v
// Project Name:
// Target Device:
// Tool versions:
// Description:
//
// Verilog Test Fixture created by ISE for module: lcdmdl
//
// Dependencies:
//
// Revision:
// Revision 1.0
// Additional Comments:
//
////////////////////////////////////////////////////////////////////////////////

module LcdMdl #(parameter L=20) (	// L is the width of the character sequence.
	input wire clk,				// Synthesis attribute PERIOD clk "50 MHz".
	input wire lcd_trgr,		// Lcd will start displaying when this line is triggered.
	input wire[L*8-1:0] lcd_code_flat,	// Character data to be displayed in wide format.
	output reg lcd_busy,		// This line will stay high when the lcd driver is busy.
	output reg lcd_rs,			// Lcd control lines...
	output wire lcd_rw,
	output reg lcd_e,
	output reg[3:0] lcd_data,	// Lcd data line.
	output wire sf_ce0			// High for full LCD access.
);

	localparam INIT=1'b0;		// Lcd states.
	localparam DISPLAY=1'b1;
	localparam K=5'h12;			// Size of K determines clock speed for lcd.

	reg[K-1:0] count=0;			// Counter for lcd clock.
	reg state=INIT;				// State variable for lcd.
	reg[3:0] init_cntr=0;		// State counter for the init state.
	reg nibble_cntr=0;			// State counter for the display state.
	wire[7:0] lcd_code[L-1:0];	// Character data to be displayed in array format.
	reg[4:0] lcd_index=0;		// Index pointer to character data.


	assign lcd_rw=0;			// No use of r/w pin.
	assign sf_ce0=1;

	genvar unpk_idx;
	generate	// This 'generate' code extends the 'lcd_code_flat' variable into the 'lcd_code' byte array.
		for(unpk_idx=0; unpk_idx<L; unpk_idx=unpk_idx+1)
		begin: unpack
			assign lcd_code[unpk_idx][7:0] = lcd_code_flat[8*unpk_idx+7:8*unpk_idx];
		end
	endgenerate

	always @(posedge clk) begin
		case(state)
		INIT:	// Starting with the initialization of the lcd.
			case(init_cntr)
			0: begin
				lcd_busy<=1'b1;		// Put the lcd into the 'busy' mode.
				lcd_rs<=1'b0;		// Command mode select.
				lcd_data<=4'h3;		// Start of the initialization sequence for 4-bit mode. (h3332)
				lcd_e<= ^count[K-1:K-2];	// Strobe the 'enable' pin with clkrate / 2^(k+2).

				if(&count)						// When 'count' is maxed,
					init_cntr<= init_cntr+ 1;	// increment state counter.

				count<= count+ 1;	// Increment 'count'.
			end

			1: begin
				lcd_data<=4'h3;
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			2: begin
				lcd_data<=4'h3;
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			3: begin
				lcd_data<=4'h2;
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			4: begin
				lcd_data<=4'h2;		// Function set. (h28)
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			5: begin
				lcd_data<=4'h8;
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			6: begin
				lcd_data<=4'h0;		// Entry mode set. (h06)
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			7: begin
				lcd_data<=4'h6;
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			8: begin
				lcd_data<=4'h0;		// Display on/off control. (h0C)
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			9: begin
				lcd_data<=4'hC;
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			10: begin
				lcd_data<=4'h0;		// Display clear. (h01)
				lcd_e<= ^count[K-1:K-2];

				if(&count)
					init_cntr<= init_cntr+ 1;

				count<= count+ 1;
			end

			11: begin
				lcd_data<=4'h1;
				lcd_e<= ^count[K-1:K-2];

				if(&count) begin	// When 'count' is maxed,
					lcd_busy<=1'b0;	// make lcd available for new data
					state<=DISPLAY;	// and put it into the 'display' state.
				end

				count<= count+ 1;
			end
			endcase

		DISPLAY:	// Display mode.
			if(lcd_trgr|lcd_busy)	// Unless one of 'lcd_trgr' or 'lcd_busy' is active, don't do anything.
				case(nibble_cntr)
				0:	// Start with the 1st nibble of input data.
					if(lcd_code[lcd_index]!=0) begin	// Check for end of data, (h0) in this case.
						lcd_busy<=1'b1;			// Put the lcd in 'busy' state.
						lcd_rs<=lcd_code[lcd_index][7:7];	// Update r/s pin with MSB of the character byte.
						lcd_data<= (lcd_code[lcd_index][7:4]& 4'h7);	// Update data pins with the remaining 3 bits of the 1st nibble.
						lcd_e<= ^count[K-1:K-2];	// Strobe the 'enable' pin.

						if(&count)				// When 'count' is maxed,
							nibble_cntr<=1'b1;	// propogate to the 2nd nibble.

						count<= count+ 1;
					end
					else begin			// If end of data (h0) achieved,
						lcd_index<=0;	// reset 'lcd_index'
						lcd_busy<=1'b0;	// and make the lcd available for new data.
					end

				1: begin	// 2nd nibble.
					lcd_data<=lcd_code[lcd_index][3:0];	// Update data pins with the 2nd nibble.
					lcd_e<= ^count[K-1:K-2];	// Strobe.

					if(&count) begin				// When 'count' is maxed,
						lcd_index<= lcd_index+ 1;	// increment index,
						nibble_cntr<=1'b0;			// revert 'nibble_cntr' back to the 1st nibble.
					end

					count<= count+ 1;
				end
				endcase
		endcase
	end
endmodule
