`timescale 1ns / 1ps

////////////////////////////////////////////////////////////////////////////////
// Company:
// Engineer:
//
// Create Date:   16:03:27 07/10/2012
// Design Name:   lcd
// Module Name:   lcd_test.v
// Project Name:
// Target Device:
// Tool versions:
// Description:
//
// Verilog Test Fixture created by ISE for module: lcd
//
// Dependencies:
//
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
//
////////////////////////////////////////////////////////////////////////////////

module main(
	input wire clk,
	input wire rot_ctr,	// Center button of rotational switch.
	output wire lcd_rs,	// Lcd control lines...
	output wire lcd_rw,
	output wire lcd_e,
	output wire[3:0] lcd_data,	// Lcd data line.
	output wire sf_ce0
);

	localparam CMD=8'h00;	// Command mask.
	localparam CHR=8'h80;	// Character mask.
	localparam CODE_LENGTH=5'h14;	// Width of the character sequence.

	reg lcd_trgr=0'b0;		// Signal for triggering the lcd.
	reg[7:0] lcd_code[CODE_LENGTH-1:0];	// Array to hold the character sequence.
	wire[CODE_LENGTH*8-1:0] lcd_code_flat;	// Character sequence compressed in a bit sequence.
	wire lcd_busy;	// 'busy' signal of the lcd.

	genvar pk_idx;
	generate	// This 'generate' code compresses the 'lcd_code' variable into the 'lcd_code_flat' bit array.
		for(pk_idx=0; pk_idx<CODE_LENGTH; pk_idx=pk_idx+1)
		begin: pack
			assign lcd_code_flat[8*pk_idx+7:8*pk_idx] = lcd_code[pk_idx][7:0];
		end
	endgenerate

	LcdMdl #(.L(CODE_LENGTH)) lcd (		// Instantiation of the lcd module.
		.clk(clk),
		.lcd_trgr(lcd_trgr),
		.lcd_code_flat(lcd_code_flat),
		.lcd_busy(lcd_busy),
		.lcd_rs(lcd_rs),
		.lcd_rw(lcd_rw),
		.lcd_e(lcd_e),
		.lcd_data(lcd_data),
		.sf_ce0(sf_ce0)
	);


	always @(posedge clk) begin
		if(lcd_busy)			// When lcd is in operation,
			lcd_trgr<=1'b0;		// reset the trigger line for later use.

		if(rot_ctr && !lcd_busy) begin	// If button is hit and lcd is NOT busy,
			lcd_code[0]<= (CMD| 8'h01);	// clear screen and display
			lcd_code[1]<= (CHR| 8'h48);	// 'H'
			lcd_code[2]<= (CHR| 8'h65);	// 'e'
			lcd_code[3]<= (CHR| 8'h6C);	// 'l'
			lcd_code[4]<= (CHR| 8'h6C);	// 'l'
			lcd_code[5]<= (CHR| 8'h6F);	// 'o'
			lcd_code[6]<= (CHR| 8'h21);	// '!'
			lcd_code[7]<= 8'h00;	// Insert zero (h00) to indicate the end of character sequence.
			lcd_trgr<=1'b1;			// Trigger the lcd.
		end
	end
endmodule
